#!perl -T

use strict;
use warnings;

use Test::More tests => 2 * 1;

require Variable::Temp;

my %syms = (
 temp => '\$',
);

for (sort keys %syms) {
 eval { Variable::Temp->import($_) };
 is $@,            '',        "import $_";
 is prototype($_), $syms{$_}, "prototype $_";
}
